#!/usr/bin/perl -w

# --------------------------------------------------------
# pukebox - A Perl based mp3 jukebox application.
#
#    (c)2000 Stuart Grimshaw. <stuart@stubbynet.co.uk>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# --------------------------------------------------------

BEGIN {
        $Common::global::gVersion = "Pukebox v0.01 Alpha (10th May 2000)";
        $Common::global::gInstallRoot = (($0 =~ m:^(.*/):)[0] || "./") . ".";
        $Common::global::gPlayList = "/var/tmp/";
        
        # You will probably need to change this to point to your MP3 dir.
        $Common::global::gMP3Path = "/home/mp3/";
        
        
}

use lib "$Common::global::gInstallRoot/lib";

use CGI qw/:standard/;	# The CPAN CGI perl module.
use Util;		# A set of usefull sub routines.
use MP3::Info;		# MP3 Tag module.
use Fcntl ':flock'; 	# import LOCK_* constants
use URI::Escape;

my(@MP3LIST);

# Open the initial directory, and search it.
my($CURDIR) = $Common::global::gMP3Path;
opendir(MP3Dir, $CURDIR);
@MP3DirList = readdir(MP3Dir);

# Loop through all the directories
&GetDirList(@MP3DirList);

# Display the menu.
$q = new CGI;

print $q->header;

print "<h1>MP3 JUKEBOX</H1>";
print $Common::global::gVersion."<P>";

# Show the menu
&ShowMenu($q->param('dir'));

# Add a song to the playlist, if needs be.
&AddToPlaylist($q->param('play')) if($q->param('play'));

# Print a footer.
print "<p><A HREF='jukebox.cgi'>Main Menu</A></p>";

# Display the current display list;
&ShowPlayList;

# ---------------------------------------------------------------
# Subroutines

# ---------------------------------------------------------------
# GetDirList

sub GetDirList {
	my(@DIR) = @_;
	my(@DIRLIST);
	my($PARENTDIR);
	
	foreach $FILE (@DIR) {
		if((-d $CURDIR.$FILE) and ($FILE ne '.') and ($FILE ne '..') ) {
			# If the file's actually a directory, recursivly call GetDirList
			$PARENTDIR = $CURDIR; 		# Set the parent directory, so it knows where it's come from.
			$CURDIR = $CURDIR.$FILE."/";	# Set the next directory.
			opendir(DIR, $CURDIR);		# Open the directory, and
			@DIRLIST = readdir(DIR);	# read in the contents.
			GetDirList(@DIRLIST);		# Call GetDirList recursivly on that directory listing.
			$CURDIR = $PARENTDIR;		# Go back to the parent directory.
		} elsif ($FILE =~ /MP3$|mp3$/s) {
			# Otherwise add it to the list.
			push(@MP3LIST, $CURDIR.$FILE);
		}
	}			
}

# ---------------------------------------------------------------
# ShowMenu

sub ShowMenu {
	my($DIR) = shift(@_);
	
	$DIR = $Common::global::gMP3Path if(!$DIR);
	
	print "<TABLE Width=75%>";
	print "<TR><TD>Artist</TD><TD>Album</TD><TD>Track</TD></TR>\n";
	
	
	foreach $THING (@MP3LIST) {
		($FOO,$ITEM) = split($DIR, $THING);
		($ITEM) = split("/", $ITEM);
		
		# Check for it in the list.
		$FOUND=0;
		foreach $FOO (@MENU) {
			if($FOO eq $ITEM) {
				$FOUND = 1;
			}
		}
		
		if($ITEM =~ /MP3$|mp3$/) {
			# Check its MP3 Tag first.
			my($MP3TAG)=get_mp3tag($DIR.$ITEM);
			
			if ((!$FOUND) and ($MP3TAG->{TITLE} ne '')) {
				$ARTIST = $MP3TAG->{ARTIST};
				$ALBUM = $MP3TAG->{ALBUM};
				$TITLE = $MP3TAG->{TITLE};
			} elsif (!$FOUND) {
				$ARTIST = "";
				$ALBUM = "";
				$TITLE = $ITEM;
			}
			
			# Change any &'s in the item to %26
			$ITEM = uri_escape($ITEM);
			$ITEM = uri_escape($ITEM, "&\'\"");

			$LINKDIR = $DIR;			# Assign $DIR to another variable, then the substitution below
			$LINKDIR = uri_escape($LINKDIR);	# doesn't effect the directory.
			$LINKDIR = uri_escape($LINKDIR, "&\'\"");
			
			print "<TR><TD>".$ARTIST."</TD><TD>".$ALBUM."</TD><TD><A HREF='jukebox.cgi?dir=".$LINKDIR."&play=".$LINKDIR.$ITEM."'>".$TITLE."</A></TD></TR>\n";
			
		} else {
			$ALBUM = $ITEM;
			$ITEM = uri_escape($ITEM);
			$ITEM = uri_escape($ITEM, "&\'\"");

			$LINKDIR = $DIR;			# Assign $DIR to another variable, then the substitution below
			$LINKDIR = uri_escape($LINKDIR);	# doesn't effect the directory.
			$LINKDIR = uri_escape($LINKDIR, "&\'\"");
			
			print "<TR><TD COLSPAN=3><A HREF='jukebox.cgi?dir=".$LINKDIR.$ITEM."/'>".$ALBUM."</A></TD></TR>\n" if !$FOUND;
		}
		push (@MENU, uri_unescape($ITEM)) if !$FOUND;
	}
	
	print "</TABLE>\n";
}

# ---------------------------------------------------------------
# AddtoPlaylist

sub AddToPlaylist {
	my($SONG) = shift(@_);
	
	open(PLAYLIST, ">>".$Common::global::gPlayList."jukebox.playlist");
	flock(PLAYLIST, LOCK_EX);
	#$SONG = &TidyFileName($SONG);
	print PLAYLIST $SONG."\n";
	flock(PLAYLIST, LOCK_UN);
	close PLAYLIST;
	
	chmod 0666,$Common::global::gPlayList."jukebox.playlist";
				
	print "<p>Added <I>".$SONG."</I> to playlist";
}

# ---------------------------------------------------------------
# TidyFileName

sub TidyFileName {
	my($FILENAME) = shift(@_);
	
	$FILENAME =~ s/"/\\\"/g;
	$FILENAME =~ s/'/\\\'/g;
	$FILENAME =~ s/&/\\\&/g;
	
	return($FILENAME);
}


# ---------------------------------------------------------------
# ShowPlayList

sub ShowPlayList {
	my($SONG);
	
	open(PLAYLIST, $Common::global::gPlayList."jukebox.playlist");
	@PLAYLIST = <PLAYLIST>;
	close PLAYLIST;

	print "<H2>Current Playlist</H2>";
	
	foreach $SONG (@PLAYLIST) {
		chomp($SONG);
		my($MP3TAG) = get_mp3tag(qq/$SONG/);
		if($MP3TAG->{TITLE}) {
			print $MP3TAG->{ARTIST}." - ".$MP3TAG->{TITLE}."<br>\n";
		} else {
			print $SONG."<br>\n";
		}
	}
}
